@@ -41,23 +41,23 @@ except Exception:
     CONDA_PACKAGE_EXTENSIONS = (CONDA_TARBALL_EXTENSION,)
 CONDA_TARBALL_EXTENSIONS = CONDA_PACKAGE_EXTENSIONS # noqa: shim for previous interface
 
-from conda.api import PackageCacheData
+from conda.api import PackageCacheData # noqa
 
-from .conda_interface import hashsum_file, md5_file, unix_path_to_win, win_path_to_unix
-from .conda_interface import PY3, iteritems
-from .conda_interface import root_dir, pkgs_dirs
-from .conda_interface import string_types
-from .conda_interface import memoized
-from .conda_interface import StringIO
-from .conda_interface import VersionOrder, MatchSpec
-from .conda_interface import cc_conda_build
-from .conda_interface import conda_43, conda_46, Dist
-from .conda_interface import context
-from .conda_interface import download, TemporaryDirectory, get_conda_channel, CondaHTTPError
+from .conda_interface import hashsum_file, md5_file, unix_path_to_win, win_path_to_unix # noqa
+from .conda_interface import PY3, iteritems # noqa
+from .conda_interface import root_dir, pkgs_dirs # noqa
+from .conda_interface import string_types # noqa
+from .conda_interface import memoized # noqa
+from .conda_interface import StringIO # noqa
+from .conda_interface import VersionOrder, MatchSpec # noqa
+from .conda_interface import cc_conda_build # noqa
+from .conda_interface import conda_43, conda_46, Dist # noqa
+from .conda_interface import context # noqa
+from .conda_interface import download, TemporaryDirectory, get_conda_channel, CondaHTTPError # noqa
 # NOQA because it is not used in this file.
-from conda_build.conda_interface import rm_rf as _rm_rf # NOQA
-from conda_build.exceptions import BuildLockError
-from conda_build.os_utils import external
+from conda_build.conda_interface import rm_rf as _rm_rf # noqa
+from conda_build.exceptions import BuildLockError # noqa
+from conda_build.os_utils import external # noqa
 
 if PY3:
     from glob import glob as glob_glob
@@ -97,6 +97,8 @@ mmap_PROT_WRITE = 0 if on_win else mmap.PROT_WRITE
 DEFAULT_SUBDIRS = {
     "linux-64",
     "linux-32",
+    "linux-s390x",
+    "linux-ppc64",
     "linux-ppc64le",
     "linux-armv6l",
     "linux-armv7l",
@@ -104,6 +106,7 @@ DEFAULT_SUBDIRS = {
     "win-64",
     "win-32",
     "osx-64",
+    "osx-arm64",
     "zos-z",
     "noarch",
 }
@@ -1209,6 +1212,7 @@ def find_recipe(path):
     if os.path.isfile(path) and os.path.basename(path) in ["meta.yaml", "conda.yaml"]:
         return os.path.dirname(path)
     results = rec_glob(path, ["meta.yaml", "conda.yaml"])
+    results = [r for r in results if os.sep + '.AppleDouble' + os.sep not in r]
     if len(results) > 1:
         base_recipe = os.path.join(path, "meta.yaml")
         if base_recipe in results:
@@ -1887,6 +1891,51 @@ def write_bat_activation_text(file_handle, m):
         file_handle.write('call "{conda_root}\\activate.bat" "{prefix}"\n'.format(
             conda_root=root_script_dir,
             prefix=m.config.build_prefix))
+    from conda_build.os_utils.external import find_executable
+    ccache = find_executable('ccache', m.config.build_prefix, False)
+    if ccache:
+        if isinstance(ccache, list):
+            ccache = ccache[0]
+        ccache_methods = {}
+        ccache_methods['env_vars'] = False
+        ccache_methods['symlinks'] = False
+        ccache_methods['native'] = False
+        if hasattr(m.config, 'ccache_method'):
+            ccache_methods[m.config.ccache_method] = True
+        for method, value in ccache_methods.items():
+            if value:
+                if method == 'env_vars':
+                    file_handle.write('set "CC={ccache} %CC%"\n'.format(ccache=ccache))
+                    file_handle.write('set "CXX={ccache} %CXX%"\n'.format(ccache=ccache))
+                elif method == 'symlinks':
+                    dirname_ccache_ln_bin = join(m.config.build_prefix, 'ccache-ln-bin')
+                    file_handle.write('mkdir {}\n'.format(dirname_ccache_ln_bin))
+                    file_handle.write('pushd {}\n'.format(dirname_ccache_ln_bin))
+                    # If you use mklink.exe instead of mklink here it breaks as it's a builtin.
+                    for ext in ('.exe', ''):
+                        # MSVC
+                        file_handle.write('mklink cl{ext} {ccache}\n'.format(ext=ext, ccache=ccache))
+                        file_handle.write('mklink link{ext} {ccache}\n'.format(ext=ext, ccache=ccache))
+                        # GCC
+                        file_handle.write('mklink gcc{ext} {ccache}\n'.format(ext=ext, ccache=ccache))
+                        file_handle.write('mklink g++{ext} {ccache}\n'.format(ext=ext, ccache=ccache))
+                        file_handle.write('mklink cc{ext} {ccache}\n'.format(ext=ext, ccache=ccache))
+                        file_handle.write('mklink c++{ext} {ccache}\n'.format(ext=ext, ccache=ccache))
+                        file_handle.write('mklink as{ext} {ccache}\n'.format(ext=ext, ccache=ccache))
+                        file_handle.write('mklink ar{ext} {ccache}\n'.format(ext=ext, ccache=ccache))
+                        file_handle.write('mklink nm{ext} {ccache}\n'.format(ext=ext, ccache=ccache))
+                        file_handle.write('mklink ranlib{ext} {ccache}\n'.format(ext=ext, ccache=ccache))
+                        file_handle.write('mklink gcc-ar{ext} {ccache}\n'.format(ext=ext, ccache=ccache))
+                        file_handle.write('mklink gcc-nm{ext} {ccache}\n'.format(ext=ext, ccache=ccache))
+                        file_handle.write('mklink gcc-ranlib{ext} {ccache}\n'.format(ext=ext, ccache=ccache))
+                    file_handle.write('popd\n')
+                    file_handle.write('set PATH={dirname_ccache_ln};{dirname_ccache};%PATH%\n'.format(
+                        dirname_ccache_ln=dirname_ccache_ln_bin,
+                        dirname_ccache=os.path.dirname(ccache)))
+                elif method == 'native':
+                    pass
+                else:
+                    print("ccache method {} not implemented")
 
 
 channeldata_cache = {}
@@ -1912,3 +1961,45 @@ def download_channeldata(channel_url):
     else:
         data = channeldata_cache[channel_url]
     return data
+
+
+def linked_data_no_multichannels(prefix):
+    """
+    Return a dictionary of the linked packages in prefix, with correct channels, hopefully.
+    cc @kalefranz.
+    """
+    from conda.core.prefix_data import PrefixData
+    from conda.models.dist import Dist
+    pd = PrefixData(prefix)
+    from conda.common.compat import itervalues
+    return {Dist.from_string(prefix_record.fn, channel_override=prefix_record.channel.name):
+                prefix_record for prefix_record in itervalues(pd._prefix_records)}
+
+
+def shutil_move_more_retrying(src, dest, debug_name):
+    log = get_logger(__name__)
+    log.info("Renaming {} directory '{}' to '{}'".format(debug_name, src, dest))
+    attempts_left = 5
+
+    while attempts_left != 0:
+        if os.path.exists(dest):
+            rm_rf(dest)
+        try:
+            log.info("shutil.move({})={}, dest={})".format(debug_name, src, dest))
+            shutil.move(src, dest)
+            if attempts_left != 5:
+                log.warning("shutil.move({}={}, dest={}) succeeded on attempt number {}".format(debug_name, src, dest,
+                                                                                                    6 - attempts_left))
+            attempts_left = 0
+        except:
+            attempts_left = attempts_left - 1
+        if attempts_left:
+            log.warning(
+                "Failed to rename {} directory, check with strace, struss or procmon. "
+                "Will sleep for 3 seconds and try again!".format(debug_name))
+            import time
+            time.sleep(3)
+        else:
+            log.error(
+                "Failed to rename {} directory despite sleeping and retrying. "
+                "This is some Windows file locking mis-bahaviour.".format(debug_name))
